#ifndef ESTIMATOR_H
#define ESTIMATOR_H

#include <math.h>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include "navsys_structs.h"
#include "communication_structs.h"
#include "RobotModel.h"
#include "Regulator.h"
#include "filter.h"

#define ODOMETRY_DIAMETER_RIGHT         51.5540		//diameter of odometry wheels in mm
#define ODOMETRY_DIAMETER_LEFT          51.5540
#define ODOMETRY_PULSES_PER_REVOLUTION  1000        //number of step in one encoder turn
#define ODOMETRY_WHEEL_DISTANCE	        335         //distance between odometry wheels, in mm

class Estimator{
    robot_state robot;
    delta delta_body;
    odometry_t odo;

    double odometry_left_prev;
    double odometry_right_prev;
    double theta_local_prev;
    double odometryspeed_left;  // speed calculated from odometry measurements
    double odometryspeed_right;
    double odo_left_filtered;   // filtered odometry measurements
    double odo_right_filtered;

    motor_speed_t motor;

	//InstanceFlag tells us whether an instance of the class has already been generated.
	static bool instanceFlag;
	//Only one instance of Estimator exists, and the reference to it is kept here.
	static Estimator* estimator;
	//Private constructor, to implement singleton-pattern.
	Estimator();

public:
    opponent_position opponent;

	//The only way to access the Estimator is through this method:
	static Estimator* getInstance();

	virtual ~Estimator();

    /* Calculate position from measurements */
    void set(odometry_t odometry);

    void setPosition(double x, double y, double theta);

    /* Set motorspeed */
    void setMotorEstimate(motor_speed_t m);

    void setInit(robot_state r);

    /* Set opponent position from info from lasertower */
    void setOpponentPosition(opponent_t opponent_info);

    void get(robot_state &rob);
    void get_delta(delta &speed);
    void get_odometry(odometry_t &o, double &odometryspeed_left, double &odometryspeed_right, double &lp_odo_l, double &lp_odo_r);
    void getMotorEstimate(motor_speed_t &m);
    void getOpponentPosition(opponent_position &opponent);
};
#endif
