#include "Plotter.h"
#define PRINT false
using namespace std;


bool Plotter::instanceFlag = false;
Plotter* Plotter::plotter = NULL;

Plotter* Plotter::getInstance(){
	if(instanceFlag==false){
		plotter=new Plotter();
		instanceFlag=true;
	}
	return plotter;
}


Plotter::Plotter(){
if(PRINT) cout<<"Construct!"<<endl;
	//Initializing the image that will display the plot.
	// 420x600 = 0.5 cm resolution (2 pixels is 1 cm)
	//if(SCALING==2)
    rawimg = cvCreateImage(cvSize(601,550),IPL_DEPTH_8U,3);

	//Input done is false as long as the user inputs balls into the problem instance.
	inputDone=false;

	directory="";
	lastSave=-1.0;
}

Plotter::~Plotter(){
	cvReleaseImage(&rawimg);
	if(PRINT) cout<<"Destroying plotter!"<<endl;
	instanceFlag = false;

}


void Plotter::plotRoute(vector<pair<double, double> > routeVector, CvScalar color){
	vector<CvPoint> cvRouteVector;
	for(unsigned int i=0;i<routeVector.size();i++){
		CvPoint cp;
		cp.x=routeVector[i].first*SCALING+OFFSET_X;
		cp.y=routeVector[i].second*SCALING+OFFSET_Y;
		cvRouteVector.push_back(cp);
	}

	for(unsigned int i=1;i<routeVector.size();i++){
		cvLine(rawimg, cvRouteVector[i-1], cvRouteVector[i], color, 1, 8);
	}

}

void Plotter::plotRouteOnImg(IplImage* plotImg, vector<pair<double, double> > routeVector, CvScalar color){
	vector<CvPoint> cvRouteVector;
	for(unsigned int i=0;i<routeVector.size();i++){
		CvPoint cp;
		cp.x=routeVector[i].first*SCALING+OFFSET_X;
		cp.y=routeVector[i].second*SCALING+OFFSET_Y;
		cvRouteVector.push_back(cp);
	}

	for(unsigned int i=1;i<routeVector.size();i++){
		cvLine(plotImg, cvRouteVector[i-1], cvRouteVector[i], color, 1, 8);
	}

}


void Plotter::plotTable(){
	//2px = 1cm
	CvPoint ul,ur,ll,lr;
	ul.x = UPPER_LEFT_X * SCALING + OFFSET_X;
	ul.y = UPPER_LEFT_Y * SCALING + OFFSET_Y;
	ll.x = LOWER_LEFT_X * SCALING + OFFSET_X;
	ll.y = LOWER_LEFT_Y * SCALING + OFFSET_Y;
	ur.x = UPPER_RIGHT_X * SCALING + OFFSET_X;
	ur.y = UPPER_RIGHT_Y * SCALING + OFFSET_Y;
	lr.x = LOWER_RIGHT_X * SCALING + OFFSET_X;
	lr.y = LOWER_RIGHT_Y * SCALING + OFFSET_Y;



	// borders of the table
	cvLine(rawimg, ul, ur, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, ll, lr, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, ul, ll, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, ur, lr, CV_RGB(0,0,0), 1, 8);

	//Drawing the starting areas
	CvPoint a1UpperRight, a1UpperLeft, a1LowerRight, a1LowerLeft, a2UpperRight, a2UpperLeft, a2LowerRight, a2LowerLeft;
	a1UpperLeft = ul;
	a1UpperRight = ul;
	a1UpperRight.x+= SCALING * STARTING_AREA_WIDTH;
	a1LowerLeft = ul;
	a1LowerLeft.y += SCALING * STARTING_AREA_WIDTH;
	a1LowerRight = a1LowerLeft;
	a1LowerRight.x += SCALING * STARTING_AREA_WIDTH;

	a2UpperRight = ur;
	a2UpperLeft = ur;
	a2UpperLeft.x -= SCALING * STARTING_AREA_WIDTH;
	a2LowerRight = ur;
	a2LowerRight.y += SCALING * STARTING_AREA_WIDTH;
	a2LowerLeft = a2LowerRight;
	a2LowerLeft.x -= SCALING * STARTING_AREA_WIDTH;

	cvLine(rawimg, a1UpperLeft, a1UpperRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a1UpperLeft, a1LowerLeft, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a1LowerLeft, a1LowerRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a1LowerRight, a1UpperRight, CV_RGB(0,0,0), 1, 8);

	cvLine(rawimg, a2UpperLeft, a2UpperRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a2UpperLeft, a2LowerLeft, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a2LowerLeft, a2LowerRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, a2LowerRight,a2UpperRight, CV_RGB(0,0,0), 1, 8);

	//Drawing the ramp
	CvPoint rampUpperLeft, rampUpperMiddle1, rampUpperMiddle2, rampUpperRight, rampLowerLeft, rampLowerMiddle1, rampLowerMiddle2, rampLowerRight;

	//TODO: Insert balls!

	rampUpperLeft = ul;
	rampUpperLeft.x += SCALING * RAMP_OFFSET_X;
	rampUpperMiddle1 = rampUpperLeft;
	rampUpperMiddle1.x += SCALING * RAMP_WIDTH;
	rampUpperMiddle2 = rampUpperMiddle1;
	rampUpperMiddle2.x += SCALING * RAMP_FLAT_WIDTH;
	rampUpperRight = rampUpperMiddle2;
	rampUpperRight.x += SCALING * RAMP_WIDTH;

	rampLowerLeft = ul;
	rampLowerLeft.y += SCALING * RAMP_HEIGHT;
	rampLowerLeft.x += SCALING * RAMP_OFFSET_X;
	rampLowerMiddle1 = rampLowerLeft;
	rampLowerMiddle1.x += SCALING * RAMP_WIDTH;
	rampLowerMiddle2 = rampLowerMiddle1;
	rampLowerMiddle2.x += SCALING * RAMP_FLAT_WIDTH;
	rampLowerRight = rampLowerMiddle2;
	rampLowerRight.x += SCALING * RAMP_WIDTH;

	cvLine(rawimg, rampUpperLeft, rampUpperRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, rampLowerLeft, rampLowerRight, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, rampUpperLeft, rampLowerLeft, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, rampUpperMiddle1, rampLowerMiddle1, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, rampUpperMiddle2, rampLowerMiddle2, CV_RGB(0,0,0), 1, 8);
	cvLine(rawimg, rampUpperRight, rampLowerRight, CV_RGB(0,0,0), 1, 8);

	//Drawing goal areas.

}

void Plotter::plotGolfBalls(){
	for(unsigned int i=0; i<ballLocations.size(); i++)
	{
		cvCircle(rawimg, ballLocations[i], 5, CV_RGB(0,0,0), -1);
	}
}


void Plotter::plotPlayingObjects(){
	//GameSetup* gs = GameSetup::getInstance();
	//vector<Point> tablePoints = gs->getTablePoints();
	//vector<Point> tableObstacles = gs->getTableObstacles();
	vector<CvPoint> tomatoLocations;
	vector<CvPoint> orangeLocations;
	vector<CvPoint> realCorns;
	vector<CvPoint> fakeCornLocations;

    CvPoint tomato1, tomato2, tomato3, tomato4, tomato5, tomato6, tomato7, tomato8, tomato9, tomato10, tomato11, tomato12, tomato13, tomato14;
	tomato1.x=150;
	tomato1.y=994;

	tomato13 = tomato1;
	tomato13.x=3000-150;

	tomato2 = tomato1;
	tomato2.y=1494;

	tomato14=tomato2;
	tomato14.x=3000-150;

	tomato3.x=600;
	tomato3.y=1244;

	tomato11=tomato3;
	tomato11.x=3000-600;

	tomato4=tomato3;
	tomato4.y=1744;

	tomato12=tomato4;
	tomato12.x=3000-600;

	tomato5.x=1050;
	tomato5.y=1494;

	tomato9=tomato5;
	tomato9.x=3000-1050;

	tomato6=tomato5;
	tomato6.y=1994;

	tomato10=tomato6;
	tomato10.x=3000-1050;

	tomato7.x=1500;
	tomato7.y=1244;

	tomato8=tomato7;
	tomato8.y=1744;

	tomatoLocations.clear();
	tomatoLocations.push_back(tomato1);
	tomatoLocations.push_back(tomato2);
	tomatoLocations.push_back(tomato3);
	tomatoLocations.push_back(tomato4);
	tomatoLocations.push_back(tomato5);
	tomatoLocations.push_back(tomato6);
	tomatoLocations.push_back(tomato7);
	tomatoLocations.push_back(tomato8);
	tomatoLocations.push_back(tomato9);
	tomatoLocations.push_back(tomato10);
	tomatoLocations.push_back(tomato11);
	tomatoLocations.push_back(tomato12);
	tomatoLocations.push_back(tomato13);
	tomatoLocations.push_back(tomato14);


    //Inserting oranges
	CvPoint orange1, orange2, orange3, orange4, orange5, orange6, orange7, orange8, orange9, orange10, orange11, orange12;
	orange1.x=orange4.x=1320;
	orange1.y=orange7.y=70;

	orange2.x=orange5.x=1375;
	orange2.y=orange8.y=145;

	orange3.x=orange6.x=1295;
	orange3.y=orange9.y=195;


	orange4.y=orange10.y=295;
	orange5.y=orange11.y=370;
	orange6.y=orange12.y=420;

	orange7.x=orange10.x=1680;
	orange8.x=orange11.x=1625;
	orange9.x=orange12.x=1705;


    orangeLocations.clear();

    orangeLocations.push_back(orange1);
    orangeLocations.push_back(orange2);
    orangeLocations.push_back(orange3);
    orangeLocations.push_back(orange4);
    orangeLocations.push_back(orange5);
    orangeLocations.push_back(orange6);

    orangeLocations.push_back(orange7);
    orangeLocations.push_back(orange8);
    orangeLocations.push_back(orange9);
    orangeLocations.push_back(orange10);
    orangeLocations.push_back(orange11);
    orangeLocations.push_back(orange12);

    CvPoint corn1, corn2, corn3, corn4, corn5, corn6, corn7, corn8, corn9, corn10, corn11, corn12, corn13, corn14, corn15, corn16, corn17, corn18;

	corn1.x=corn2.x=corn3.x=150;
	corn4.x=corn5.x=corn6.x=600;
	corn7.x=corn8.x=1050;
	corn9.x=corn10.x=1500;
	corn11.x=corn12.x=1950;
	corn13.x=corn14.x=corn15.x=2400;
	corn16.x=corn17.x=corn18.x=2850;

	corn1.y=corn16.y=744;
	corn2.y=corn7.y=corn11.y=corn17.y=1244;

	corn3.y=corn8.y=corn12.y=corn18.y=1744;
	corn4.y=corn13.y=994;
	corn5.y=corn9.y=corn14.y=1494;
	corn6.y=corn10.y=corn15.y=1994;

    realCorns.push_back(corn1);
    realCorns.push_back(corn2);
    realCorns.push_back(corn3);
    realCorns.push_back(corn4);
    realCorns.push_back(corn5);
    realCorns.push_back(corn6);
    realCorns.push_back(corn7);
    realCorns.push_back(corn8);
    realCorns.push_back(corn9);
    realCorns.push_back(corn10);
    realCorns.push_back(corn11);
    realCorns.push_back(corn12);
    realCorns.push_back(corn13);
    realCorns.push_back(corn14);
    realCorns.push_back(corn15);
    realCorns.push_back(corn16);
    realCorns.push_back(corn17);
    realCorns.push_back(corn18);




    plotOranges(orangeLocations);
    //plotTomatoes(tomatoLocations);
    plotRealCorns(realCorns);
    //plotFakeCorns(fakeCornLocations);

}

void Plotter::plotPartly(plott_data_t pd){
	if(PRINT) cout<<"Plot starting"<<endl;

    cvNamedWindow("Visualizer2D", CV_WINDOW_AUTOSIZE);


	cvSet(rawimg, CV_RGB(255,255,255),NULL);
    plotTable();
    plotPlayingObjects();
    plotRobotPosition(pd.robot_xpos,pd.robot_ypos,pd.robot_angle,false);

    	CvPoint wp1;
	wp1.x=200*SCALING;
	wp1.y=1000*SCALING;

	cvCircle(rawimg, wp1, 100*SCALING, CV_RGB(0,255,0), 1);

        CvPoint wp2;
	wp2.x=400*SCALING;
	wp2.y=1000*SCALING;

	cvCircle(rawimg, wp2, 100*SCALING, CV_RGB(0,255,0), 1);

        CvPoint wp3;
	wp3.x=760*SCALING;
	wp3.y=1500*SCALING;

	//cvCircle(rawimg, wp3, 100*SCALING, CV_RGB(0,255,0), 1);

//	CvPoint wp4;
//	wp4.x=240*SCALING;
//	wp4.y=1260*SCALING;

//	    CvPoint a4;
//    a4.x = round(wp4.x + 100*SCALING*cos(-M_PI/2));
//    a4.y = round(wp4.y + 100*SCALING*sin(-M_PI/2));

//	cvCircle(rawimg, wp4, 100*SCALING, CV_RGB(0,255,0), 1);
	//cvLine(rawimg, wp4, a4,  CV_RGB(0,255,0), 1);


    //Plotting current time on simulator image.
    CvFont myFont;
    cvInitFont(&myFont,CV_FONT_HERSHEY_SIMPLEX,0.7f,0.7f,0,1,8);

    stringstream text;
    text << "ROBOT-> X:" <<pd.robot_xpos <<" Y:" << pd.robot_xpos <<" ANGLE:" << (pd.robot_angle*180/(M_PI));
    cvPutText(rawimg, text.str().c_str(), cvPoint(10,450), &myFont , cvScalar(0x255,0x0,0x0) );

    text.str("");
    text << "ENEMY-> DISTANCE:" <<pd.enemy_dist <<" ANGLE:" << (pd.enemy_angle*180/(M_PI));
    cvPutText(rawimg, text.str().c_str(), cvPoint(10,470), &myFont , cvScalar(0x0,0x0,0x255) );

    text.str("");
    text << "BEACON A-> DISTANCE:" <<pd.beaconA_dist <<" ANGLE:" << (pd.beaconA_angle*180/(M_PI));
    cvPutText(rawimg, text.str().c_str(), cvPoint(10,490), &myFont , cvScalar(0x0,0x0,0x0) );

    text.str("");
    text << "BEACON B-> DISTANCE:" <<pd.beaconB_dist <<" ANGLE:" << (pd.beaconB_angle*180/(M_PI));
    cvPutText(rawimg, text.str().c_str(), cvPoint(10,510), &myFont , cvScalar(0x0,0x0,0x0) );

    text.str("");
    text << "BEACON C-> DISTANCE:" <<pd.beaconC_dist <<" ANGLE:" << (pd.beaconC_angle*180/(M_PI));
    cvPutText(rawimg, text.str().c_str(), cvPoint(10,530), &myFont , cvScalar(0x0,0x0,0x00) );



		//For allowing animated image.
		IplImage* temp = (IplImage*) cvClone(rawimg);
		cvShowImage("Visualizer2D", temp);

		int key;
		key = cvWaitKey(10);
		cvReleaseImage(&temp);
}


void Plotter::plotRobotPosition(double x, double y, double w, bool enemy){
	x*=SCALING;
	x+=OFFSET_X;
	y*=SCALING;
	y+=OFFSET_Y;

	CvPoint robot;
	robot.x=round(x);
	robot.y=round(y);

    CvPoint anglePoint;
    anglePoint.x = round(x + 192.5*SCALING*cos(w));
    anglePoint.y = round(y + 192.5*SCALING*sin(w));


	if(enemy==false)
	{
		cvCircle(rawimg, robot, 192.5*SCALING, CV_RGB(0,0,255), 1);
		cvLine(rawimg, robot, anglePoint,  CV_RGB(0,0,255), 1);
	}
	else
	{
		cvCircle(rawimg, robot, 192.5*SCALING, CV_RGB(255,0,0), 1);
		cvLine(rawimg, robot, anglePoint,  CV_RGB(255,0,0), 1);
	}
}



void Plotter::plotTomatoes(std::vector<CvPoint> tomatoLocations){
	for(unsigned int i=0; i<tomatoLocations.size(); i++)
	{
		tomatoLocations[i].x*=SCALING;
		tomatoLocations[i].x+=OFFSET_X;
		tomatoLocations[i].y*=SCALING;
		tomatoLocations[i].y+=OFFSET_Y;
	}

	for(unsigned int i=0; i<tomatoLocations.size(); i++)
	{
		cvCircle(rawimg, tomatoLocations[i], 50*SCALING, CV_RGB(255,0,0), -1);
	}

}

void Plotter::plotOranges(std::vector<CvPoint> orangeLocations){
	for(unsigned int i=0; i<orangeLocations.size(); i++)
	{
		orangeLocations[i].x*=SCALING;
		orangeLocations[i].x+=OFFSET_X;
		orangeLocations[i].y*=SCALING;
		orangeLocations[i].y+=OFFSET_Y;
	}


	for(unsigned int i=0; i<orangeLocations.size(); i++)
	{
		cvCircle(rawimg, orangeLocations[i], 50*SCALING, CV_RGB(255,125,0), -1);
	}

}

void Plotter::plotRealCorns(std::vector<CvPoint> realCornLocations){
	for(unsigned int i=0; i<realCornLocations.size(); i++)
	{
		realCornLocations[i].x*=SCALING;
		realCornLocations[i].x+=OFFSET_X;
		realCornLocations[i].y*=SCALING;
		realCornLocations[i].y+=OFFSET_Y;
	}


	for(unsigned int i=0; i<realCornLocations.size(); i++)
	{
		cvCircle(rawimg, realCornLocations[i], 25*SCALING, CV_RGB(0,0,0), 0);
	}

}

void Plotter::plotFakeCorns(std::vector<CvPoint> FakeCornLocations){
	for(unsigned int i=0; i<FakeCornLocations.size(); i++)
	{
		FakeCornLocations[i].x*=SCALING;
		FakeCornLocations[i].x+=OFFSET_X;
		FakeCornLocations[i].y*=SCALING;
		FakeCornLocations[i].y+=OFFSET_Y;
	}


	for(unsigned int i=0; i<FakeCornLocations.size(); i++)
	{
		cvCircle(rawimg, FakeCornLocations[i], 25*SCALING, CV_RGB(0,0,0), -1);
	}

}
