#ifndef PLOTTER_H
#define PLOTTER_H
#include "image.h"
/*
#include <cv.h>
#include <highgui.h>
#include <cxmisc.h>
*/
#include <iostream>
#include <sstream>
#include <cstdio>
#include <ctime>
#include <vector>
#include <utility>
#include <ios>
#include <fstream>
#include<sys/stat.h>
#include<sys/types.h>
#include<fstream>
#include <time.h>
#include <string>
#include <sys/statfs.h>
#include <sys/types.h>
#include <math.h>
//Defining some of the points on the playing table
#define UPPER_LEFT_X 0
#define UPPER_LEFT_Y 0
#define UPPER_RIGHT_X 3000
#define UPPER_RIGHT_Y 0
#define LOWER_LEFT_X 0
#define LOWER_LEFT_Y 2122
#define LOWER_RIGHT_X 3000
#define LOWER_RIGHT_Y 2122

#define STARTING_AREA_WIDTH 500
#define STARTING_AREA_HEIGHT 500

#define RAMP_WIDTH 500
#define RAMP_HEIGHT 500
#define RAMP_FLAT_WIDTH 519
#define RAMP_OFFSET_X 740

#define OFFSET_X 0
#define OFFSET_Y 0
#define SCALING 0.2

typedef struct{
    double robot_xpos;
    double robot_ypos;
    double robot_angle;
    double enemy_dist;
    double enemy_angle;
    double beaconA_dist;
    double beaconA_angle;
    double beaconB_dist;
    double beaconB_angle;
    double beaconC_dist;
    double beaconC_angle;
}plott_data_t;


class Plotter
{
	private:
		//This is the only instance of plotter that will exist.
		static Plotter* plotter;

		//Tells us if a plotter has already been initialized.
		static bool instanceFlag;

		//The picture that the plotter displays.
		IplImage* rawimg;

		//Locations of all the balls the user has plotted onto the image.
		std::vector<CvPoint> ballLocations;

		bool inputDone;

		//The individual to be plotted, and that can be affected by user input.
		//Individual individual;

		//These are all the objects that have been removed from the table.
		//std::vector<Point> removedPoints;

		double lastSave;

		std::string directory;

		//std::vector<waypoint_struct_ai> waypoints;

	public:

		void resetPlotter();

		//The only way to access the plotter-instance.
		static Plotter* getInstance();


		//Basic destructor
		~Plotter();


		//Plots the paths resulting from different evolutions in different colors. Also plots the current position of the robot
		//in the dynamic simulation and the expected fitness of the current plan. Can also save plots at certain intervals, if
		//saveInterval is set to a positive, non-zero integer.
		void plotPartly(plott_data_t pd);
		//Allows the user to insert objects on the plotting image before the EA is run.
		void insertObjects();

		//Plots golf balls at the posisitions given in the input vector.
		void plotGolfBalls();

		void plotTomatoes(std::vector<CvPoint> tomatoLocations);

		void plotOranges(std::vector<CvPoint> orangeLocations);

		void plotRealCorns(std::vector<CvPoint> realCornLocations);

		void plotFakeCorns(std::vector<CvPoint> fakeCornLocations);

		//Called when the user clicks the image to plot a new ball.
		void addGolfBall(CvPoint ballLocation);

		//returns all the balls on the table.
		std::vector<CvPoint> getBallLocations();

		//Plots a series of images representing saved static solutions to a given table setup.
		void plotAndSave(std::vector<std::vector<std::pair <double, double> > >& routeVector,std::vector<std::string> imgName,
				std::string saveFolder, std::vector<std::string> fitness);

		//Pressing left mouse button on a playing element removes this element from the table setup.
		void leftMousePressed(double x,double y);

		//Pressing right mouse button on the table, adds the most recently removed playing element to this spot.
		void rightMousePressed(double x,double y);

		//Plots all the objects on the table according to the current game setup.
		void plotPlayingObjects();

		//Sets waypoints for plotted: Intended for experimenting with WP-generation.
		//void setWaypoints(std::vector<waypoint_struct_ai> waypointsVector){
		//	waypoints=waypointsVector;
		//}

  private:

		//Plots the robot's current position on the playing table when performing dynamic plotting. Used for plotting both own robot and enemy.
		void plotRobotPosition(double x, double y, double w, bool enemy);

		//Basic constructor is private, making this a singleton.
		Plotter();

		//Generates a plot of the playing table.
		void plotTable();

		//Plots the route the robot has followed, in the given color.
		void plotRoute(std::vector<std::pair <double, double> > routeVector,CvScalar color);

		//Same as plotRoute, but plots on a specified image instead of rawimg.
		void plotRouteOnImg(IplImage* plotImg, std::vector<std::pair<double, double> > routeVector, CvScalar color);

};

void mousePressed(int event, int x, int y, int flags, void* param);

#endif //PLOTTER_H
