#include "Regulator.h"

double error_r_integ = 0;
double error_psi_integ = 0;
bool waypoint_reached = false;
int wp_teller = 0;

bool Regulator::instanceFlag = false;
Regulator* Regulator::regulator = NULL;

Regulator* Regulator::getInstance(){
	if(instanceFlag==false){
		regulator = new Regulator();
		instanceFlag=true;
	}
	return regulator;
}

Regulator::Regulator(){
    next_wp.x_local = 0;
    next_wp.y_local = 0;
    next_wp.theta_local = 0;
    next_wp.should_correct_angle = 0;
    next_wp.top_of_hill = 0;
    next_wp.circle = 0;
    next_wp.stop_on_wp = 0;
    next_wp.r_body = 0;
    next_wp.psi_body = 0;

    output.u_body = 0;
    output.w_body = 0;
    }

Regulator::~Regulator(){
    }

void Regulator::set(robot_state &robot, output_speed &output, waypoint_list &ai_waypoints){

   // wp_teller = 0;
    next_waypoint(ai_waypoints, robot, wp_teller);

   // cout << "regulator angle:" << this->next_wp.psi_body << endl;
    cout << "regulator dist:" << this->next_wp.r_body << endl;

    int reverse_limit = 100;

/*
    if (wp_teller>=2) {
        cout << " stop on waypoint " << endl;
        this->next_wp.r_body = 0;
        this->next_wp.psi_body = 0;
        this->output.u_body = 0;
        this->output.w_body = 0;
        return;
    }
*/

    /// Check if waypoint is reached. If not: run pid(). ///
    if (!waypoint_reached && fabs(next_wp.r_body)<CIRCLE_OF_ACCEPTANCE && fabs(output.u_body)<10){
        this->next_wp.r_body = 0;
        this->output.u_body = 0;
        usleep(100);

        if (!next_wp.should_correct_angle) {
            wp_teller+=1;
            sendWaypointReached(); /// Sending WP_reached to AI

            if (next_wp.stop_on_wp) { //wp_teller>=1) {
                cout << " stop on waypoint " << endl;
                this->next_wp.r_body = 0;
                this->next_wp.psi_body = 0;
                this->output.u_body = 0;
                this->output.w_body = 0;
            }
            // wp_teller+=1;

            cout << " WP reached! " << endl;
            cout << " ******************* " << endl;
            //cout << "r: " << next_wp.r_body << endl;
            //cout << "psi: " << next_wp.psi_body << endl;
        }
        else {
            waypoint_reached = true;
        }
    }

    else if(!waypoint_reached && fabs(next_wp.r_body)>=CIRCLE_OF_ACCEPTANCE){
        if (next_wp.circle == 1) { // drive along a circle, right
            next_wp.psi_body = next_wp.psi_body/2;
            pid(robot, true);
            output.w_body += (ODOMETRY_WHEEL_DISTANCE/(2*R))*output.u_body;
        }
        else if (next_wp.circle == -1) { // drive along a circle, left
            next_wp.psi_body = next_wp.psi_body/2;
            pid(robot, true);
            output.w_body -= (ODOMETRY_WHEEL_DISTANCE/(2*R))*output.u_body;
        }

        if ((fabs(next_wp.r_body)<=reverse_limit && fabs(next_wp.psi_body)>(M_PI/2)) ||
            ((next_wp.reverse==true) &&  (fabs(next_wp.r_body)>reverse_limit))){ // reverse
                next_wp.r_body = -fabs(next_wp.r_body);
                pid(robot, true);
        }

        else if ((next_wp.reverse==true) && (fabs(next_wp.r_body)<=reverse_limit) && fabs(next_wp.psi_body)<=(M_PI/2)){
            /* if in reverse-mode and still close enough to a waypoint in front of the robot */
            cout << " reverse + forward " << endl;
            pid(robot,true);
        }

        else if (next_wp.circle == 0) {
            if (fabs(next_wp.r_body)>reverse_limit && fabs(next_wp.psi_body)>=0.3) {
                pid(robot, false); // runs only the controller for rotation
            }
            else  {
                pid(robot, true); // runs both controllers
            }
        }
    }

    else if (waypoint_reached && next_wp.should_correct_angle && fabs(next_wp.r_body)<(CIRCLE_OF_ACCEPTANCE*5)){
        /* waypoint reached and ready to correct new angle */
        next_wp.psi_body = next_wp.theta_body;
        cout << "rotation angle:" << this->next_wp.psi_body << endl;
        if (fabs(next_wp.psi_body)<ANGLE_OF_ACCEPTANCE && fabs(output.w_body<30)){
            cout << " angle reached! " << endl;
            cout << " ***************** " << endl;
            this->next_wp.psi_body = 0;
            this->output.w_body = 0;
            waypoint_reached = false;
            //wp_teller += 1;
            sendWaypointReached(); /// Send WP_reached to AI
            if (next_wp.stop_on_wp) {
                cout << " stop on waypoint (angle) " << endl;
                this->next_wp.r_body = 0;
                this->next_wp.psi_body = 0;
                this->output.u_body = 0;
                this->output.w_body = 0;
            }
        }
        else pid(robot, false); // only rotation
    }

    else if (waypoint_reached && next_wp.should_correct_angle && fabs(next_wp.r_body)>=(CIRCLE_OF_ACCEPTANCE*5)){
        pid(robot, true);
    }
    else {
        pid(robot,false);
    }
}


void Regulator::get(output_speed &controller_output, int &circle, bool &top_of_hill, double &e){
    controller_output.u_body = output.u_body;
    controller_output.w_body = output.w_body;
    circle = next_wp.circle;
    top_of_hill = next_wp.top_of_hill;
    e = next_wp.r_body;
    }

void Regulator::get_waypoint(waypoint &wp){
    wp = next_wp;
}

void Regulator::pid(robot_state &robot, bool dist_controller ){

    double error_angle = next_wp.psi_body;
    double error_dist = next_wp.r_body;

    //cout << "pid: error angle: " << error_angle << endl;
    //cout << "pid: error dist: " << error_dist << endl;

    static double Kp_dist = 0.3;
    static double Kd_dist = 10;
    static double Ki_dist = 0;
    static double u_dist_max = 50.0;
    static double u_dist_min = -50.0;
    static double u_dist_slew = 0.1;
    static double fri_max_dist = 38;
    static double fri_min_dist = 8;

    static double Kp_angle = 0.2;
    static double Kd_angle = 16;
    static double Ki_angle = 0;
    static double u_angle_max = 45.0;
    static double u_angle_min = -45.0;
    static double u_angle_slew = 0.1;

    static PID *pid_dist = NULL; /* posistion controller -- distance */
    static PID *pid_angle = NULL; /* position controller -- angle */

    if(!dist_controller){
        Kp_angle = 0.3;
        Kd_angle = 8;
        Ki_angle = 0.0012;
    }
    if(next_wp.reverse){
        u_dist_max = 40.0;
        u_dist_min = -40.0;
        Kp_dist = 0.008;
        Kd_dist = 10;
        Ki_dist = 0.00001;
        Kp_angle = 0.3;
        Kd_angle = 20;
        Ki_angle = 0.0001;
    }
    if(next_wp.top_of_hill){
        u_dist_max = 70.0;
        u_dist_min = 0.0;
        u_angle_max = 40.0;
        u_angle_min = -40.0;
        u_dist_slew = 0.00001;
        fri_max_dist = 50;
        fri_min_dist = 45;
    }

    /* PID init */
    if(pid_dist == NULL){
        /* Setting PID-parametres */
        pid_dist = new PID(Kp_dist, Ki_dist, Kd_dist, u_dist_slew, u_dist_max, u_dist_min);
        pid_angle = new PID(Kp_angle, Ki_angle, Kd_angle, u_angle_slew, u_angle_max, u_angle_min);
        pid_dist->fri_max = fri_max_dist;
        pid_dist->fri_min = fri_min_dist;
        pid_angle->fri_max = 36;
        pid_angle->fri_min = 8;

        pid_dist->integral_max =  100;
        pid_dist->integral_min = -100;
        pid_angle->integral_max =  100;
        pid_angle->integral_min = -100;
    }

    /// Set set point in mm
    pid_dist->setSetpoint(error_dist);
    pid_angle->setSetpoint((ODOMETRY_WHEEL_DISTANCE/2.0)*error_angle);

    /** Run controllers **/
    if (dist_controller == true) { // run translation and rotation
        cout << " \t DRIVING " << endl;
            this->output.u_body = round(pid_dist->run(0));
            this->output.w_body = round(pid_angle->run(0));
    }
    else { // dist_controller == false --- run rotation only
        cout << " \t rotating " << endl;
        this->output.u_body = 0;
        this->output.w_body = round(pid_angle->run(0));
    }
}

void Regulator::next_waypoint(waypoint_list ai_waypoints, robot_state &robot, int &wp_teller){

    // Information from waypoint-list from AI
	next_wp.x_local = ai_waypoints.wp_list[wp_teller].x;
    next_wp.y_local = ai_waypoints.wp_list[wp_teller].y;
    next_wp.theta_local = ai_waypoints.wp_list[wp_teller].theta;
    next_wp.should_correct_angle = ai_waypoints.wp_list[wp_teller].should_correct_angle;
    next_wp.top_of_hill = ai_waypoints.wp_list[wp_teller].top_of_hill;
    next_wp.circle = ai_waypoints.wp_list[wp_teller].circle;
    next_wp.stop_on_wp = ai_waypoints.wp_list[wp_teller].stop_on_wp;
    next_wp.reverse = ai_waypoints.wp_list[wp_teller].reverse;

	// next waypoint in robot-coordinates
	double x_robot_wp = cos(robot.theta_local)*(next_wp.x_local-robot.x_local) + sin(robot.theta_local)*(next_wp.y_local-robot.y_local);
	double y_robot_wp = -sin(robot.theta_local)*(next_wp.x_local-robot.x_local) + cos(robot.theta_local)*(next_wp.y_local-robot.y_local);

	// Distance and direction to next waypoint
	next_wp.r_body = sqrt(pow(x_robot_wp,2) + pow(y_robot_wp,2));
	next_wp.psi_body = atan2(y_robot_wp,x_robot_wp);	//in radians: -PI til PI

    //heading of wp in robot-coordinates - only if should_correct_angle = true.
    if (next_wp.should_correct_angle == true) {
        next_wp.theta_body = next_wp.theta_local - robot.theta_local;
        angleWrap(next_wp.theta_body);			// theta between -pi and pi
    }
    else {
        next_wp.theta_body = 0;
    }
}

void angleWrap(double &theta){ 		// keeps the angle between -pi and pi
	if (theta>M_PI) {
	    theta = theta-2*M_PI;
	    }
	if (theta<-M_PI) {
	    theta = theta+2*M_PI;
	    }
	}
