#ifndef REGULATOR_H
#define REGULATOR_H

#include <math.h>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <signal.h>

#include "RobotModel.h"
#include "navsys_structs.h"
#include "pid.h"
#include "Estimator.h"

using namespace std;

#define CIRCLE_OF_ACCEPTANCE      20     // mm
#define ANGLE_OF_ACCEPTANCE        0.1   // rad
#define R			             260     // radius of circle to drive along around obstacles

class Regulator{

    waypoint next_wp;
    output_speed output;

    /* controller */
    void pid(robot_state &robot, bool dist_controller);

    /* Compute next waypoint in robot-coordinates */
    void next_waypoint(waypoint_list ai_waypoints, robot_state &robot, int &wp_teller);


    //InstanceFlag tells us whether an instance of the class has already been generated.
	static bool instanceFlag;
	//Only one instance of Estimator exists, and the reference to it is kept here.
	static Regulator* regulator;
	//Private constructor, to implement singleton-pattern.
	Regulator();

public:
	//The only way to access the Estimator is through this method.
	static Regulator* getInstance();

	virtual ~Regulator();

    void set(robot_state &robot, output_speed &output, waypoint_list &ai_waypoints);
    void get(output_speed &controller_output, int &circle, bool &top_of_hill, double &e);

    void get_waypoint(waypoint &wp);
};

void angleWrap(double &theta);

#endif
