#include "RobotModel.h"

using namespace std;

Regulator *regulator = Regulator::getInstance();
delta hastighet;
odometry_t odo;
motor_speed_t motor_u;

Filter filter_l_speed, filter_r_speed;
waypoint_list ai_waypoints;

double odo_lp_left;
double odo_lp_right;

bool RobotModel::instanceFlag = false;
RobotModel* RobotModel::robotmodel = NULL;

RobotModel* RobotModel::getInstance(){
	if(instanceFlag==false){
        robotmodel = new RobotModel(0, 0, 0, 0, 0);
		instanceFlag=true;
	}
	return robotmodel;
}

RobotModel::RobotModel(double x, double y, double theta, double u, double w){

	robot_estimate.x_local = x;
	robot_estimate.y_local = y;
	robot_estimate.theta_local = theta;
	robot_estimate.u_body = u;
	robot_estimate.w_body = w;

    output.u_body = 0;
    output.w_body = 0;

    motorspeed.motor1 = 0;
    motorspeed.motor2 = 0;

    u_l_desired = 0;
    u_r_desired = 0;

    odometryspeed_left = 0;
    odometryspeed_right = 0;

    Estimator *estimator = Estimator::getInstance();
    estimator->setInit(robot_estimate);

    u_l_prev = 0;
    u_r_prev = 0;
}

RobotModel::RobotModel(){
	robot_estimate.x_local = 0;
	robot_estimate.y_local = 0;
	robot_estimate.theta_local = 0;
	robot_estimate.u_body = 0;
	robot_estimate.w_body = 0;
    output.u_body = 0;        //nsket hast body
    output.w_body = 0;
    motorspeed.motor1 = 0;
    motorspeed.motor2 = 0;
    u_l_desired = 0;            //nsket hast langs hjul(odo)
    u_r_desired = 0;
    odometryspeed_left = 0;        //hast fra odometry
    odometryspeed_right = 0;

}

RobotModel::~RobotModel(){
}

void RobotModel::get_desired_output(output_speed &s,  double &l_d, double &r_d){
    s.u_body = output.u_body;
    s.w_body = output.w_body;
    l_d = u_l_desired;
    r_d = u_r_desired;
}

void RobotModel::getMotorSpeed(motor_speed_ref_t &m){
    m.motor1 = motorspeed.motor1;
    m.motor2 = motorspeed.motor2;
}

void RobotModel::updateModel(){

    // get position estimate
    Estimator *estimator = Estimator::getInstance();
    estimator->get(this->robot_estimate);
    estimator->get_odometry(odo, odometryspeed_left, odometryspeed_right, odo_lp_left, odo_lp_right);
    estimator->getMotorEstimate(motor_u);

    cout << "position: x: " << this->robot_estimate.x_local << "y: " << this->robot_estimate.y_local << " theta: " << this->robot_estimate.theta_local << endl;

    //filter_l_speed.LPfilter(odometryspeed_left);
    //filter_r_speed.LPfilter(odometryspeed_right);

    // get waypoint list if new waypoints have arrived
    if (waypoints_received) {
        getWaypoints(ai_waypoints);
    }
    else return;

    //getWpList_test(ai_waypoints);       /// for testing without ai

    //setting desired output:
    int circle;         // if driving on a circle
    bool top_of_hill;   // if driving up hill
    double e;           // distance to waypoint, only on hill.
    regulator->set(robot_estimate, output, ai_waypoints);
    regulator->get(output, circle, top_of_hill, e);  //
    /**
    * NB!!! Output is pwm-signal to motors.
    **/

    // Desired output to motors computed from translation and rotation output.
    u_r_desired = output.u_body - output.w_body; //(ODOMETRY_WHEEL_DISTANCE/2)*output.w_body;
    u_l_desired = output.u_body + output.w_body; //(ODOMETRY_WHEEL_DISTANCE/2)*output.w_body;

    cout << "hast fra odo: odometryspeed_left: " << odometryspeed_left << " odometruspeed_right: " << odometryspeed_right << endl;
    cout << "pdrag: u_body: " << output.u_body << " w_body: " << output.w_body << endl;
    cout << "pdrag1: u_l: " << u_l_desired << " u_r: " << u_r_desired << endl;

    int max_output;
    if (circle != 0) max_output = 80;
    else max_output = 70;

    // limits output to +-max_output, adjusting the other output proportionally
    if (fabs(u_r_desired) >= fabs(u_l_desired)) {
        if (fabs(u_r_desired)>max_output && u_r_desired<0) {
            u_l_desired = -max_output*(u_l_desired/u_r_desired);
            u_r_desired = -max_output;
        }
        else if (u_r_desired>max_output) {
            u_l_desired = max_output*(u_l_desired/u_r_desired);
            u_r_desired = max_output;
        }
    }
    if (fabs(u_l_desired) > fabs(u_r_desired)) {
        if (fabs(u_l_desired)>max_output && u_l_desired<0) {
            u_r_desired = -max_output*(u_r_desired/u_l_desired);
            u_l_desired = -max_output;
        }
        else if (u_l_desired>max_output) {
            u_r_desired = max_output*(u_r_desired/u_l_desired);
            u_l_desired = max_output;
        }
    }

    cout << "pdrag2: u_l: " << u_l_desired << " u_r: " << u_r_desired << endl;

    /* If friction to overcome */
       int friction = 43;       // output to overcome the friction
       int fric_increase = 3;   // increase if "friction" is not enough
    if (!top_of_hill && fabs(odometryspeed_left)<40 && fabs(u_l_desired)>3){
        cout << " ...friction, if left... " << endl;
        if (u_l_desired<friction && u_l_desired>0) {
            u_l_desired = friction;
        }
        else if (u_l_desired>-friction && u_l_desired<0) {
            u_l_desired = -friction;
        }
        else if (u_l_prev>=friction) {
            u_l_desired = u_l_prev+fric_increase;
        }
        else if (u_l_prev<=-friction) {
            u_l_desired = u_l_prev-fric_increase;
        }
    }
    if (!top_of_hill && fabs(odometryspeed_right)<100 && fabs(u_r_desired)>3){
        cout << " ...friction, if right... " << endl;
        if (u_r_desired<friction && u_r_desired>0) {
            u_r_desired = friction;
        }
        else if (u_r_desired>-friction && u_r_desired<0) {
            u_r_desired = -friction;
        }
        else if (u_r_prev>=friction) {
            u_r_desired = u_r_prev+fric_increase;
        }
        else if (u_r_prev<=-friction) {
            u_r_desired = u_r_prev-fric_increase;
        }
    }

    cout << "pdrag3: u_l: " << u_l_desired << " u_r: " << u_r_desired << endl;

    /* limits change in output */
    double max_increase = 3.0;    // max acceleration
    if (fabs(u_l_desired)>friction && fabs(u_l_desired-u_l_prev) > max_increase) {
        if ((u_l_desired>0) && (u_l_desired-u_l_prev > 0)) {
            u_l_desired = u_l_prev + max_increase;
        }
        else if ((u_l_desired<0) && (u_l_desired-u_l_prev > 0)){
            u_l_desired = u_l_prev + max_increase;
        }
        else if ((u_l_desired<0) && (u_l_desired-u_l_prev < 0)) {
            u_l_desired = u_l_prev - max_increase;
        }
        else if ((u_l_desired>0) && (u_l_desired-u_l_prev < 0)) {
            u_l_desired = u_l_prev - max_increase;
        }
    }
    if (fabs(u_r_desired)>friction &&  fabs(u_r_desired-u_r_prev) > max_increase) {
        if ((u_r_desired>0) && (u_r_desired-u_r_prev > 0)) {
            u_r_desired = u_r_prev + max_increase;
        }
        else if ((u_r_desired<0) && (u_r_desired-u_r_prev > 0)){
            u_r_desired = u_r_prev + max_increase;
        }
        else if ((u_r_desired<0) && (u_r_desired-u_r_prev < 0)) {
            u_r_desired = u_r_prev - max_increase;
        }
        else if ((u_r_desired>0) && (u_r_desired-u_r_prev < 0)) {
            u_r_desired = u_r_prev - max_increase;
        }
    }

    u_l_prev = u_l_desired;
    u_r_prev = u_r_desired;

    cout << "pdrag4: u_l: " << u_l_desired << " u_r: " << u_r_desired << endl;

    // if only rotation
    if (fabs(output.u_body)<5) {
        if (u_l_desired>10 && u_r_desired<-10){    // rotate right, u_r_desired<0
           if (odometryspeed_left+odometryspeed_right>10) {
               u_r_desired-=5;
               u_l_desired-=5;
           }
           else if (odometryspeed_left+odometryspeed_right<-10) {
               u_l_desired+=5;
               u_r_desired+=5;
           }
        }
        else if (u_l_desired<-10 && u_r_desired>10) {
            if (odometryspeed_right+odometryspeed_left>10) {
                u_l_desired+=5;
                u_r_desired+=5;
            }
            else if (odometryspeed_right+odometryspeed_left<-10) {
                u_r_desired-=5;
                u_l_desired-=5;
            }
        }
    }

    if (u_r_desired>127) u_r_desired = 127;
    else if (u_r_desired<-126) u_r_desired = -126;
    if (u_l_desired>127) u_l_desired = 127;
    else if (u_l_desired<-126) u_l_desired = -126;

    motorspeed.motor1 = (char) -round(u_r_desired);
    motorspeed.motor2 = (char) round(u_l_desired);

    cout << "motorspeed: u_l: " << (int)motorspeed.motor2 << " u_r: " << (int)motorspeed.motor1 << endl;

/*    // sjekker om fremdrift spinner
    int speed_diff = 200;
    double speed_from_left_motor = motor_u.motor2*DRIVE_WHEEL_DIAMETER*M_PI/MOTOR_GEAR; //hast fra motor i mm/s
    double speed_from_right_motor = motor_u.motor1*DRIVE_WHEEL_DIAMETER*M_PI/MOTOR_GEAR;
    if (fabs(odometryspeed_left-speed_from_left_motor) > speed_diff){ // hva m strm? sette hast t 0 v for mye strm?
        motorspeed.motor2 = 0;
        motorspeed.motor1 = 0;
    }
    if (fabs(odometryspeed_right-speed_from_right_motor) > speed_diff){
        motorspeed.motor2 = 0;
        motorspeed.motor1 = 0;
    }*/

/// Sending output to motor ///
    sendMotorSpeed(&motorspeed);
}

void RobotModel::stopModel(){
	motorspeed.motor1 = 0;
	motorspeed.motor2 = 0;
	sendMotorSpeed(&motorspeed);
	}

void RobotModel::getWaypoints(waypoint_list &ai_waypoints){
    getWpList(&ai_waypoints);
}

void RobotModel::getWpList_test(waypoint_list &ai_waypoints){

        ai_waypoints.wp_list[0].x = 2700;
        ai_waypoints.wp_list[0].y = 900;
        ai_waypoints.wp_list[0].theta = 0;
        ai_waypoints.wp_list[0].should_correct_angle = false;
        ai_waypoints.wp_list[0].top_of_hill = false;
        ai_waypoints.wp_list[0].circle = false;
        ai_waypoints.wp_list[0].stop_on_wp = false;
        ai_waypoints.wp_list[0].reverse = false;

        ai_waypoints.wp_list[1].x = 2000;
        ai_waypoints.wp_list[1].y = 1400;
        ai_waypoints.wp_list[1].theta = 0;
        ai_waypoints.wp_list[1].should_correct_angle = false;
        ai_waypoints.wp_list[1].top_of_hill = false;
        ai_waypoints.wp_list[1].circle = false;
        ai_waypoints.wp_list[1].stop_on_wp = true;
        ai_waypoints.wp_list[1].reverse = false;

        /*

        ai_waypoints.wp_list[2].x = 760;
        ai_waypoints.wp_list[2].y = 1500;
        ai_waypoints.wp_list[2].theta = 0;
        ai_waypoints.wp_list[2].should_correct_angle = false;
        ai_waypoints.wp_list[2].top_of_hill = false;
        ai_waypoints.wp_list[2].circle = false;
        ai_waypoints.wp_list[2].stop_on_wp = true;
        ai_waypoints.wp_list[2].reverse = false;

        ai_waypoints.wp_list[3].x = 240;
        ai_waypoints.wp_list[3].y = 1260;
        ai_waypoints.wp_list[3].theta = 0;
        ai_waypoints.wp_list[3].should_correct_angle = false;
        ai_waypoints.wp_list[3].top_of_hill = false;
        ai_waypoints.wp_list[3].circle = false;
        ai_waypoints.wp_list[3].stop_on_wp = true;
        ai_waypoints.wp_list[3].reverse = false;

        ai_waypoints.wp_list[4].x = 500;
        ai_waypoints.wp_list[4].y = 1000;
        ai_waypoints.wp_list[4].theta = 0;
        ai_waypoints.wp_list[4].should_correct_angle = false;
        ai_waypoints.wp_list[4].top_of_hill = false;
        ai_waypoints.wp_list[4].circle = true;
        ai_waypoints.wp_list[4].stop_on_wp = false;
        ai_waypoints.wp_list[4].reverse = false;*/
    }

