#ifndef ROBOT_MODEL_H
#define ROBOT_MODEL_H

#include <math.h>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <signal.h>
#include <sys/time.h>

#include "communication.h"
#include "Estimator.h"
#include "Regulator.h"
#include "communication_structs.h"
#include "navsys_structs.h"
#include "messageQueue.h"
#include "posix_messages.h"
#include "main.h"
#include "pid.h"
#include "filter.h"

using namespace std;

#define T                       0.01 	//  time between each sample
#define MOTOR_GEAR             18
#define DRIVE_WHEEL_DIAMETER   56       //  mm
#define DRIVE_WHEEL_DISTANCE  300       //  mm

class RobotModel{

    output_speed output;
    motor_speed_ref_t motorspeed;

    double u_l_desired;
    double u_r_desired;
    double odometryspeed_left;
    double odometryspeed_right;

    double u_l_prev;
    double u_r_prev;

    //InstanceFlag tells us whether an instance of the class has already been generated.
	static bool instanceFlag;
	//Only one instance of RobotModel exists, and the reference to it is kept here.
	static RobotModel* robotmodel;
	//Private constructor, to implement singleton-pattern.
	RobotModel();
    RobotModel(double x, double y, double psi, double u, double w);

public:
    robot_state robot_estimate;

	//The only way to access the RobotModel is through this method.
	static RobotModel* getInstance();

	virtual ~RobotModel();

    void get_desired_output(output_speed &s,double &l_d, double &r_d);
    void getMotorSpeed(motor_speed_ref_t &m);

    void updateModel();
	void stopModel();

    void getWaypoints(waypoint_list &ai_waypoints);
    void getWpList_test(waypoint_list &wp_list);
	};

#endif
