#include "main.h"

using namespace std;

odometry_t odometry;
robot_state r;
output_speed s;
motor_speed_ref_t motor;
Plotter * gui;
plott_data_t gui_data;
pthread_t GUIthread;

robot_position robotpos;
opponent_position opponentpos;

Regulator *reg = Regulator::getInstance();

starting_position startingposition;
//int t;
//int tid;
Logger navlog;

motor_speed_t m;
waypoint next_wp;
delta speed;

double odo_delta_l;
double odo_delta_r;

double odo_prev_l;
double odo_prev_r;

double l_d;         // desired output left wheel
double r_d;         // desired output right wheel
double odometryspeed_left;  // velocity from odometry
double odometryspeed_right;

double lp_odo_l;
double lp_odo_r;

int main() {

    communicationInit();

    /** Wait until a starting posistion is received from AI **/
    while (true){
        pthread_mutex_lock(&startMutex);
        if (start_received){
            cout << "start_received" << endl;
            break;
        }
        pthread_mutex_unlock(&startMutex);
        usleep(50000);
    }

    init();

    Estimator *e = Estimator::getInstance();
    RobotModel *robot = RobotModel::getInstance();

    /** set startingposition after received from AI */
    getStartingPosition(&startingposition);
    r.x_local = startingposition.x;
    r.y_local = startingposition.y;
    r.theta_local = startingposition.psi;
    r.u_body = 0;
    r.w_body = 0;
    e->setInit(r);

    cout << "start: " << startingposition.x << ", " << startingposition.y << " , " << startingposition.psi << endl;

    struct timeval time_start, time_end;

    while(1) {
        /*To get the time between each iteration of the while loop*/
        gettimeofday(&time_start, NULL);

        /* this is the method that runs the controller and computes output */
        robot->updateModel();

        e->get_delta(speed);
        e->get_odometry(odometry, odometryspeed_left,odometryspeed_right, lp_odo_l, lp_odo_r);
        e->getMotorEstimate(m);
        e->get(r);
        robot->get_desired_output(s,l_d, r_d);
        robot->getMotorSpeed(motor);

        reg->get_waypoint(next_wp);
        e->getOpponentPosition(opponentpos);

        robotpos.x = round(r.x_local);
        robotpos.y = round(r.y_local);
        robotpos.psi = r.theta_local;
        robotpos.speed = r.u_body;
        robotpos.timestamp = odometry.timestamp;

        sendRobotPositionAi(&robotpos);
        sendOpponentPositionAi(&opponentpos);


        double M1 = m.motor1*DRIVE_WHEEL_DIAMETER*M_PI/MOTOR_GEAR;
        double M2 = m.motor2*DRIVE_WHEEL_DIAMETER*M_PI/MOTOR_GEAR;

        cout << "main: odo l:" << odometry.left << " r: " << odometry.right  << endl;
        cout << " pos: X: " << robotpos.x << " Y: " << robotpos.y << " rot: " << robotpos.psi << endl;

        /** Logging **/
        navlog.logEventFloat(2, 26, (double)odometry.left, (double)odometry.right,
                                speed.odo_left, speed.odo_right, l_d,r_d, odometryspeed_left,odometryspeed_right,
                                s.u_body, s.w_body, (double)m.motor2,(double)m.motor1,
                                M2, M1, (double)motor.motor2,-(double)motor.motor1,
                                lp_odo_l,lp_odo_r, r.x_local, r.y_local, r.theta_local,
                                (double)next_wp.x_local, (double)next_wp.y_local, next_wp.theta_local,
                                next_wp.r_body, next_wp.psi_body
                                );

        /* For plotting of GUI-interface when testing without AI */
        //if (t%10 == 0){
        //    gui_data.robot_xpos = r.x_local;
        //    gui_data.robot_ypos = r.y_local;
        //    gui_data.robot_angle = r.theta_local;
        //}

        gettimeofday(&time_end, NULL);
        cout << "Time delta[us]: " << (time_end.tv_usec-time_start.tv_usec) << endl;
        usleep(10000);
        /* 1 tid = 10ms */
    }
    return 0;
}


void init(){
    odometryspeed_left = 0;
    odometryspeed_right = 0;

    odo_prev_l = 0;
    odo_prev_r = 0;

    /*Create singals handlers*/
    signal(SIGINT, catch_int);

    resetOdometry();

    RobotModel *robot = RobotModel::getInstance();
    Estimator *e = Estimator::getInstance();

    /*Create plotter*/
    //gui = Plotter::getInstance();
    //gui->plotPartly(gui_data);

    /*Create thread*/
    //if(pthread_create(&GUIthread, NULL, plotGUIThread, NULL)){
    //    perror("guiThread");
    //    cout << "GUI Thread [error]" <<endl;
    //    exit(1);
    //}
    //else{
    //    cout << "GUI Thread [ok]" << endl;
    //}

    if(navlog.startLog("log/test"))
        cout << "LOGGER ERROR " << endl;
    else
        cout << "LOGGER OK! "<< endl;
}


void catch_int(int sig_num){

	signal(SIGINT, catch_int);

	//communicationExit();

	cout << " CTRL+C EXIT NAVSYS" << endl;

	RobotModel *robot = RobotModel::getInstance();
	robot->stopModel();

	exit(1);
}

extern "C" void * plotGUIThread(void * arg){
    while(1){
        gui->plotPartly(gui_data);
        usleep(100000);
    }
    return NULL;
}
