#include "messageQueue.h"

using namespace std;


MessageQueue::MessageQueue(string queueName){
    struct mq_attr attr;
    // Attributes
	attr.mq_maxmsg = MAX_POSIX_MESSAGE_QUEUE_LENGTH;
	attr.mq_msgsize = sizeof(posix_message_t);
	attr.mq_flags   = 0;

	if(queueName[0] != '/') {
		throw invalid_argument("The supplied argument is not a valid file descriptor / channel. Must start with '/'");
	} else if(queueName.length()<3) {
		throw  invalid_argument("File descriptor / channel name must have length>=3");
	}
    mq_name = queueName;

	int oflag_send = O_WRONLY|O_NONBLOCK|O_CREAT;
	int oflag_receive =O_RDONLY;
    int oflag_receive_noblock =O_RDONLY|O_NONBLOCK;
	// Open the queue, and create it if it's not already created.
	send_handle = mq_open(queueName.c_str(),oflag_send,PMODE,&attr);
	if (send_handle == -1) {
		printf("mq_open failed for %s. Are you root?",queueName.c_str());
		exit(1);
	}

    receive_handle = mq_open(queueName.c_str(),oflag_receive,PMODE,&attr);
	if (receive_handle == -1) {
		printf("mq_open failed for %s. Are you root?",queueName.c_str());
		exit(1);
	}

	receive_noblock_handle = mq_open(queueName.c_str(),oflag_receive_noblock,PMODE,&attr);
	if (receive_handle == -1) {
		printf("mq_open failed for %s. Are you root?",queueName.c_str());
		exit(1);
	}

}

MessageQueue::~MessageQueue(){
    mq_close(send_handle);
    mq_close(receive_handle);
}

int MessageQueue::receive(posix_message_t * msg){
	int num_bytes_received = mq_receive(receive_handle, (char *)msg, sizeof(posix_message_t), 0);
    if (num_bytes_received == -1)
    {
        perror("mq_receive failed (-1).");
		return -1;
    }
    else if(num_bytes_received != (msg->length+POSIX_MESSAGE_HEADER_SIZE))
    {
		printf("Error-> number of byte received %d\n",num_bytes_received);
		return -1;
	}
	// Success.
	return 0;
}

int MessageQueue::send(posix_message_t * msg){
    /*check length of messages*/
    int msg_length = msg->length+POSIX_MESSAGE_HEADER_SIZE;
    if( msg_length < 0|| (unsigned int)msg_length > sizeof(posix_message_t)){
        printf("Error-> Wrong message length: %d\n",msg->length);
    }

    if (mq_send(send_handle, (char *)msg, msg_length , msg->priority) == -1){
    printf("Failed to send posix message id: %i length:%i\n", msg->id,msg->length);
		return -1;
	}
	// Success.
	return 0;
}

void MessageQueue::sendStruct(short id,short priority,short length,char * data){
    /*building message*/
    posix_message_t msg;
    msg.id = id;
    msg.priority = priority;
    msg.length = length;
    for(int i=0; i< length; i++)
        msg.data[i] = data[i];

    /*send message to Queue*/
    send(&msg);
}
void MessageQueue::sendCommand(short id,short priority){
    /*building message*/
    posix_message_t msg;
    msg.id = id;
    msg.priority = priority;
    msg.length = 0;

    /*send message to Queue*/
    send(&msg);
}

int MessageQueue::clear(){
    posix_message_t msg;
    int i=0;
    while(mq_receive(receive_noblock_handle, (char *)&msg, sizeof(posix_message_t), 0)!=-1){
        i++; //count messages in queue
    }
    printf("%d messages was cleared in %s queue",i,mq_name.c_str());
    return i;
}

void MessageQueue::remove(){
    if(mq_unlink(mq_name.c_str())) perror("mq_unlink");
}
