#ifndef __MESSAGE_QUEUE_H__
#define __MESSAGE_QUEUE_H__
#include <iostream>
#include <string>
#include <stdexcept>
#include <errno.h>
#include <vector>
#include <ostream>
#include <cstdio>
#include <cstdlib>

extern "C"{
#include <error.h>
#include <fcntl.h>           /* For O_* constants */
#include <sys/stat.h>        /* For mode constants */
#include <mqueue.h>
}

#define PMODE 0666
#define MAX_POSIX_MESSAGE_SIZE 128 // max is cylinder_config @ 108 bytes.
#define MAX_POSIX_MESSAGE_QUEUE_LENGTH 100
#define POSIX_MESSAGE_HEADER_SIZE 6 //sizeof(short)*3


typedef struct{
    /*header*/
	short id;                           // Message ID
	short priority;						// Message priority.
	short length;						// Data length.
	/*data*/
	char data[MAX_POSIX_MESSAGE_SIZE];	// Message data.
}posix_message_t;


class MessageQueue{
  private:
    mqd_t send_handle;
    mqd_t receive_handle;
    mqd_t receive_noblock_handle;
    std::string mq_name;
  public:
    MessageQueue(std::string queueName);
    ~MessageQueue();
    int receive(posix_message_t * msg);
    int send(posix_message_t * msg);
    void sendStruct(short id,short priority,short length,char * data);
    void sendCommand(short id,short priority);
    int clear();
    void remove();

};
#endif

