#ifndef PID_H_INCLUDED
#define PID_H_INCLUDED

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <iostream>
#include <pthread.h>
#include <sys/time.h>

#include "Estimator.h"
#include "RobotModel.h"
#include "Regulator.h"

#define PID_DEFAULT_INTEGRAL_MAX +INFINITY
#define PID_DEFAULT_INTEGRAL_MIN -INFINITY

extern pthread_mutex_t pid_controller_mutex; //

class PID{
    private:
        void init();
    public:
        double p_gain;   /* 'P' proportional gain               */
        double i_gain;   /* 'I' integral gain                   */
        double d_gain;   /* 'D' derivative gain                 */

        double fri_min;  /* min-value for extra friction output */
        double fri_max;  /* max-value for extra friction output */
        double setpt;    /* set point                           */
        double min;      /* minimum output value                */
        double max;      /* maximum output value                */
        double slew;     /* maximum slew rate on output         */
        double integral;
        double integral_max;
        double integral_min;
        double last_error;
        double last_output;

        pthread_mutex_t setpoint_mutex;

        PID(double Kp, double Kd, double Ki, double slew, double max, double min);
        ~PID();

        void setSetpoint(double setpt);
        double run(double PV);
};

/* Argumentstruct to motor_speed_thread */
struct pid_thread_args{
	int frequency;
	PID *pid_1,
		*pid_2;
    output_speed* output;
    bool dist_controller;
};

/* Trdfunksjonen */
void* motor_speed_thread(void* args);

#endif // PID_H_INCLUDED
